@@ -39,9 +39,9 @@ setMethod("Ops", signature(e1 = "bigz", e2 = "mpfr"),
 	  function(e1, e2) callGeneric(.bigz2mpfr(e1), e2))
 
 setMethod("Ops", signature(e1 = "mpfr", e2 = "bigq"),
-	  function(e1, e2) callGeneric(e1, .bigq2mpfr(e2)))
+	  function(e1, e2) callGeneric(e1, ..bigq2mpfr(e2, pmax(.getPrec(e1), 128L))))
 setMethod("Ops", signature(e1 = "bigq", e2 = "mpfr"),
-	  function(e1, e2) callGeneric(.bigq2mpfr(e1), e2))
+	  function(e1, e2) callGeneric(..bigq2mpfr(e1, pmax(.getPrec(e1), 128L)), e2))
 
 
 setMethod("Logic", signature(e1 = "mpfr", e2 = "mpfr"),
@@ -65,7 +65,7 @@ setMethod("Logic", signature(e1 = "numeric", e2 = "mpfr"),
 }
 .mpfr.negative <- function(x) .Call(Rmpfr_minus, x)
 
-setMethod("Arith",signature(e1 = "mpfr", e2="missing"),
+setMethod("Arith", signature(e1 = "mpfr", e2="missing"),
 	  function(e1,e2) {
 	    switch(.Generic,
 		   "+" = e1,
@@ -169,22 +169,27 @@ setMethod("Arith", signature(e1 = "mpfrArray", e2 = "mpfr"),
 	      e1
 	  })
 
-## "macro-like	encapsulation
-.Arith.num.mpfr <- function(x,y, FUN)
-    .Call(if(is.integer(x)) "Arith_i_mpfr" else "Arith_d_mpfr",
-	  x,y, .Arith.codes[FUN])
 
-.Arith.mpfr.num <- function(x,y, FUN)
-    .Call(if(is.integer(y)) "Arith_mpfr_i" else "Arith_mpfr_d",
-	  x, y, .Arith.codes[FUN])
+## "macro-like	encapsulation"
+.Arith.num.mpfr <- function(x,y, FUN) {
+    ff <- if(is.integer(x)) Arith_i_mpfr else Arith_d_mpfr
+    .Call(ff, x,y, .Arith.codes[FUN])
+}
 
-.Compare.num.mpfr <- function(x,y, FUN)
-    .Call(if(is.integer(x)) "Compare_mpfr_i" else "Compare_mpfr_d",
-	  y, x, .Compare.codesRev[FUN])
+.Arith.mpfr.num <- function(x,y, FUN) {
+    ff <- if(is.integer(y)) Arith_mpfr_i else Arith_mpfr_d
+    .Call(ff, x, y, .Arith.codes[FUN])
+}
 
-.Compare.mpfr.num <- function(x,y, FUN)
-    .Call(if(is.integer(y)) "Compare_mpfr_i" else "Compare_mpfr_d",
-	  x, y, .Compare.codes[FUN])
+.Compare.num.mpfr <- function(x,y, FUN) {
+    ff <- if(is.integer(x)) Compare_mpfr_i else Compare_mpfr_d
+    .Call(ff, y, x, .Compare.codesRev[FUN])
+}
+
+.Compare.mpfr.num <- function(x,y, FUN) {
+    ff <- if(is.integer(y)) Compare_mpfr_i else Compare_mpfr_d
+    .Call(ff, x, y, .Compare.codes[FUN])
+}
 
 setMethod("Arith", signature(e1 = "array", e2 = "mpfr"),# incl "mpfrArray"
 	  function(e1, e2) {
